@@ -22,8 +22,8 @@ class Experimento:
 
     def __init__(self, nombre_archivo, dimension_caja=[100, 100], cantidad_filas=10, cantidad_columnas=10,
                  procesar_tiempo_archivo_irregular=False, puntos_importantes=None, objeto_a_promediar=1,
-                 tamanio_intervalo_para_promedio=100, genera_matriz_recurrencia=False,
-                 tamanio_cilindros=24, div_cilindros=2):
+                 tamanio_intervalo_para_promedio=100, genera_matriz_recurrencia=False, regiones_preferidas=None,
+                 distancia_max_para_volumenes=24, division_de_dist_max=2):
         self.nombre_archivo = nombre_archivo
         print("NOMBRE_ARCHIVO:", self.nombre_archivo, "\n TIPO_ARCHIVO:", type(self.nombre_archivo))
         self.dimension_caja = dimension_caja
@@ -111,7 +111,9 @@ class Experimento:
             self.objeto_relevante_3 = puntos_importantes[2]
             self.objeto_relevante_4 = puntos_importantes[3]
 
-        self.radios_cilindros = None
+        self.regiones_preferidas = regiones_preferidas
+
+        self.radios_volumenes = None
 
         self.distancia_a_objeto_promediado = None
 
@@ -129,6 +131,7 @@ class Experimento:
         self.posiciones_y = arreglo_posicion_y
         self.tiempo = arreglo_tiempo
         self.samples = arreglo_samples
+
         print("Se encuentra la columna consecuences: ", self.archivo_consecuencias)
         if self.archivo_consecuencias is True:
             self.consecuencias = arreglo_consecuencias
@@ -185,11 +188,17 @@ class Experimento:
 
         self.matriz_recurrencia_region_cuadriculada = None
         self.matriz_cuadriculada = None
+        self.matriz_cuadriculada_regiones_preferidas = None
+
+        self.visitas_regiones_preferidas = None
+        self.visitas_regiones_no_preferidas = None
 
         self.cantidad_posiciones_volumen_1 = None
         self.cantidad_posiciones_volumen_2 = None
         self.cantidad_posiciones_volumen_3 = None
         self.cantidad_posiciones_volumen_4 = None
+        self.distancia_max_para_volumenes = distancia_max_para_volumenes
+        self.division_de_dist_max = division_de_dist_max
         self.radios_volumenes = None
 
         print("Construcción de series de tiempo y de posición.")
@@ -219,6 +228,8 @@ class Experimento:
         self.calcula_regiones_cuadriculadas()
         if genera_matriz_recurrencia is True:
             self.calcula_matrices_de_recurrencia()
+        if self.matriz_cuadriculada_regiones_preferidas is not None:
+            self.genera_matriz_cuadriculada_con_regiones_preferidas()
 
     def verifica_intervalo_tiempo(self):
         arreglo_tiempo = self.tiempo
@@ -447,13 +458,25 @@ class Experimento:
 
         print(self.matriz_cuadriculada)
 
+    def genera_matriz_cuadriculada_con_regiones_preferidas(self):
+        matriz_cuadriculada_plana = np.flipud(self.matriz_cuadriculada).flatten()
+        self.visitas_regiones_preferidas = matriz_cuadriculada_plana[self.regiones_preferidas]
+        self.visitas_regiones_no_preferidas = np.delete(matriz_cuadriculada_plana, self.regiones_preferidas)
+
+        resultado_visitas_regiones_preferidas = sum(self.visitas_regiones_preferidas)
+        resultado_visitas_regiones_no_preferidas = sum(self.visitas_regiones_no_preferidas)
+
+        arreglo_regiones_preferidas = resultado_visitas_regiones_no_preferidas * np.ones(len(matriz_cuadriculada_plana))
+        arreglo_regiones_preferidas[self.regiones_preferidas] = resultado_visitas_regiones_preferidas
+
+        self.matriz_cuadriculada_regiones_preferidas = \
+            np.reshape(arreglo_regiones_preferidas, (self.cantidad_filas, self.cantidad_columnas))
+
     def calcula_matrices_de_recurrencia(self):
 
         self.obtener_indice_regiones_objeto()
 
         cantidad_tiempos = len(self.tiempo)
-        cantidad_filas = self.cantidad_filas
-        cantidad_columnas = self.cantidad_columnas
 
         arreglo_region_cuadriculada = self.arreglo_region_cuadriculada
 
@@ -515,24 +538,24 @@ class Experimento:
             dist_3 = fa.calcula_distancia_entre_puntos(self.objeto_relevante_3[0], x_pos, self.objeto_relevante_3[1],
                                                        y_pos)
             dist_4 = fa.calcula_distancia_entre_puntos(self.objeto_relevante_4[0], x_pos, self.objeto_relevante_4[1],
-                                                           y_pos)
+                                                       y_pos)
             if dist_1 < tamanio:
                 cantidad_posiciones_volumen_1 = fa.modificar_lista_para_cilindros(cantidad_posiciones_volumen_1,
                                                                                   radios_figuras,
-                                                                                  dist_1)
+                                                                                  dist_1,
+                                                                                  tipo=tipo)
             if dist_2 < tamanio:
                 cantidad_posiciones_volumen_2 = fa.modificar_lista_para_cilindros(cantidad_posiciones_volumen_2,
                                                                                   radios_figuras,
-                                                                                  dist_2)
+                                                                                  tipo=tipo)
             if dist_3 < tamanio:
                 cantidad_posiciones_volumen_3 = fa.modificar_lista_para_cilindros(cantidad_posiciones_volumen_3,
                                                                                   radios_figuras,
-                                                                                  dist_3)
+                                                                                  tipo=tipo)
             if dist_4 < tamanio:
                 cantidad_posiciones_volumen_4 = fa.modificar_lista_para_cilindros(cantidad_posiciones_volumen_4,
                                                                                   radios_figuras,
-                                                                                  dist_4)
-
+                                                                                  tipo=tipo)
 
         print("Cantidad de posiciones para volumen 1:")
         print(cantidad_posiciones_volumen_1)
@@ -1326,6 +1349,90 @@ class Experimento:
 
             ax.view_init(azim=rot_grafica_3d)
 
+        elif grafica == 'region_cuadricula_3d_regiones_preferidas':
+
+            print("Entro a regionCuadricula3d, regiones preferidas")
+
+            rot_grafica_3d = rotacion_grafica_3d
+
+            objeto_1_x = self.objeto_relevante_1[0]
+            objeto_1_y = self.objeto_relevante_1[1]
+
+            objeto_2_x = self.objeto_relevante_2[0]
+            objeto_2_y = self.objeto_relevante_2[1]
+
+            objeto_3_x = self.objeto_relevante_3[0]
+            objeto_3_y = self.objeto_relevante_3[1]
+
+            objeto_4_x = self.objeto_relevante_4[0]
+            objeto_4_y = self.objeto_relevante_4[1]
+
+            if getattr(sys, 'frozen', False):
+                os.environ['BASEMAPDATA'] = os.path.join(os.path.dirname(sys.executable), 'mplot3d')
+
+            data_array = np.array(np.flipud(self.matriz_cuadriculada_regiones_preferidas))
+
+            print(data_array)
+
+            ax = fig.add_subplot(111, projection='3d')
+
+            ax.view_init(azim=rot_grafica_3d % 360, elev=0)
+
+            x_data, y_data = np.meshgrid(np.arange(data_array.shape[1]),
+                                         np.arange(data_array.shape[0]))
+
+            x_data = x_data.flatten()
+            y_data = y_data.flatten()
+            z_data = data_array.flatten()
+
+            tamanio_horizontal = self.dimension_caja[0] / self.cantidad_columnas
+            tamanio_vertical = self.dimension_caja[1] / self.cantidad_filas
+
+            objeto_1_x_pos_matriz_cuadriculada = int(objeto_1_x // tamanio_horizontal)
+            objeto_1_y_pos_matriz_cuadriculada = int(objeto_1_y // tamanio_vertical)
+
+            objeto_2_x_pos_matriz_cuadriculada = int(objeto_2_x // tamanio_horizontal)
+            objeto_2_y_pos_matriz_cuadriculada = int(objeto_2_y // tamanio_vertical)
+
+            objeto_3_x_pos_matriz_cuadriculada = int(objeto_3_x // tamanio_horizontal)
+            objeto_3_y_pos_matriz_cuadriculada = int(objeto_3_y // tamanio_vertical)
+
+            objeto_4_x_pos_matriz_cuadriculada = int(objeto_4_x // tamanio_horizontal)
+            objeto_4_y_pos_matriz_cuadriculada = int(objeto_4_y // tamanio_vertical)
+
+            print([objeto_1_x_pos_matriz_cuadriculada],
+                  [objeto_2_x_pos_matriz_cuadriculada],
+                  [objeto_3_x_pos_matriz_cuadriculada],
+                  [objeto_4_x_pos_matriz_cuadriculada])
+            print([objeto_1_y_pos_matriz_cuadriculada],
+                  [objeto_2_y_pos_matriz_cuadriculada],
+                  [objeto_3_y_pos_matriz_cuadriculada],
+                  [objeto_4_y_pos_matriz_cuadriculada])
+
+            ax.bar3d(x_data,
+                     y_data,
+                     np.zeros(len(z_data)),
+                     1, 1, z_data)
+
+            if 0 < rot_grafica_3d <= 90:
+                ax.xaxis._axinfo['juggled'] = (0, 0, 0)
+                ax.yaxis._axinfo['juggled'] = (1, 1, 1)
+                ax.zaxis._axinfo['juggled'] = (2, 2, 2)
+            elif 90 < rot_grafica_3d <= 180:
+                ax.xaxis._axinfo['juggled'] = (0, 0, 0)
+                ax.yaxis._axinfo['juggled'] = (0, 1, 2)
+                ax.zaxis._axinfo['juggled'] = (0, 2, 1)
+            elif 180 < rot_grafica_3d <= 270:
+                ax.xaxis._axinfo['juggled'] = (1, 0, 2)
+                ax.yaxis._axinfo['juggled'] = (0, 1, 2)
+                ax.zaxis._axinfo['juggled'] = (2, 2, 2)
+            elif 270 < rot_grafica_3d <= 360:
+                ax.xaxis._axinfo['juggled'] = (1, 0, 2)
+                ax.yaxis._axinfo['juggled'] = (1, 1, 1)
+                ax.zaxis._axinfo['juggled'] = (1, 2, 0)
+
+            ax.view_init(azim=rot_grafica_3d)
+
         return fig
 
 
